package org.herac.tuxguitar.player.impl.midiport.winmm;

import java.util.ArrayList;
import java.util.List;

public class MidiSystem{	
	
	private static final String JNI_LIBRARY_NAME = new String("tuxguitar-winmm-jni");
	
	static{
		JNILibraryLoader.loadLibrary(JNI_LIBRARY_NAME);
	}		
	
	private int instance;
	private List ports;
	private MidiPortImpl connection;

	public MidiSystem() {
		this.instance = malloc();
		this.ports = new ArrayList();
		this.connection = null;
	}
	
	public void finalize(){
		if(this.instance != 0 ){
			this.free(this.instance);
			this.instance = 0;
		}
	}

	public List findPorts(){
		this.ports.clear();
		if(this.instance != 0){
			this.findPorts(this.instance);
		}
		return this.ports;
	}
	
	public void openPort(MidiPortImpl port){
		if(this.instance != 0){
			this.openPort(this.instance, port.getDevice());
			this.connection = port;
		}
	}
	
	public void closePort(){
		if(this.instance != 0){
			this.closePort(this.instance);
			this.connection = null;
		}
	}
	
	public void noteOn(int channel,int note,int velocity){
		if(this.instance != 0){
			this.noteOn(this.instance, channel, note, velocity);
		}
	}
	
	public void noteOff(int channel,int note,int velocity){
		if(this.instance != 0){
			this.noteOff(this.instance, channel, note, velocity);
		}
	}

	public void controlChange(int channel,int control,int value){
		if(this.instance != 0){
			this.controlChange(this.instance, channel, control, value);
		}
	}
	
	public void programChange(int channel,int program){
		if(this.instance != 0){
			this.programChange(this.instance, channel, program);
		}
	}
	
	public void pitchBend(int channel,int value){
		if(this.instance != 0){
			this.pitchBend(this.instance, channel, value);
		}
	}
	
	public MidiPortImpl getConnection(){
		return this.connection;
	}
	
	protected void addPort(String name,int device){
		this.ports.add(new MidiPortImpl(this,name,device));
	}	
	
	private native int malloc();
	
	private native void free(int instance);	
	
	protected native void open(int instance,String device);
	
	protected native void close(int instance);	
	
	protected native void findPorts(int instance);
	
	protected native void openPort(int instance,int device);
	
	protected native void closePort(int instance);
	
	protected native void noteOn(int instance,int channel,int note,int velocity);
	
	protected native void noteOff(int instance,int channel,int note,int velocity);

	protected native void controlChange(int instance,int channel,int control,int value);
	
	protected native void programChange(int instance,int channel,int program);
	
	protected native void pitchBend(int instance,int channel,int value);

}
