package org.herac.tuxguitar.gui.system.config.items;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ToolBar;
import org.herac.tuxguitar.gui.TuxGuitar;
import org.herac.tuxguitar.gui.helper.SyncThread;
import org.herac.tuxguitar.gui.system.config.TGConfigEditor;
import org.herac.tuxguitar.gui.system.config.TGConfigKeys;

public class LanguageOption extends Option{
	protected boolean initialized;
	protected Button[] buttons;
    protected String[] languages;
    protected String language;
    protected String selected;
    
    public LanguageOption(TGConfigEditor configEditor,ToolBar toolBar,final Composite parent){
        super(configEditor,toolBar,parent,TuxGuitar.getProperty("settings.config.language"));
        this.initialized = false;
    }
    
    public void createOption(){		
		getToolItem().setText(TuxGuitar.getProperty("settings.config.language"));	
		getToolItem().setImage(TuxGuitar.instance().getIconManager().getOptionLanguage());
		getToolItem().addSelectionListener(this);
		
		showLabel(getComposite(),SWT.FILL,SWT.CENTER,SWT.TOP | SWT.LEFT | SWT.WRAP,SWT.BOLD,0,TuxGuitar.getProperty("settings.config.language.choose"));	
		
		Composite composite = new Composite(getComposite(),SWT.NONE);
		composite.setLayout(new GridLayout());
		composite.setLayoutData(getTabbedData());
		
		this.languages = TuxGuitar.instance().getLanguageManager().getLanguages();
		if(this.languages != null){
			this.buttons = new Button[this.languages.length];
			for(int i = 0;i < this.buttons.length;i++){
				this.buttons[i] = new Button(composite,SWT.RADIO); 
				this.buttons[i].setData(this.languages[i]);
				this.buttons[i].setText(TuxGuitar.getProperty( ("locale." + this.languages[i]) ) );
			}
		}
		this.loadConfig();
    }                  
        
    protected void loadConfig(){
    	new Thread(new Runnable() {		
			public void run() {
				LanguageOption.this.language = getConfig().getStringConfigValue(TGConfigKeys.LANGUAGE);
				if(LanguageOption.this.language == null || LanguageOption.this.language.trim().length() == 0){
					LanguageOption.this.language = TuxGuitar.instance().getLanguageManager().getLanguage();
				}
				if(LanguageOption.this.languages != null){
					for(int i = 0;i < LanguageOption.this.languages.length; i ++){
						if(LanguageOption.this.languages[i].equals(LanguageOption.this.language)){
							final int index = i;
							new SyncThread(new Runnable() {					
								public void run() {
									if(!isDisposed()){
										LanguageOption.this.buttons[index].setSelection(true);
										LanguageOption.this.initialized = true;
										LanguageOption.this.pack();
									}
								}
							}).start();						
							return;
						}
					}
					LanguageOption.this.initialized = true;
			    	new SyncThread(new Runnable() {		
						public void run() {
							LanguageOption.this.pack();
						}
			    	}).start();
				}
			}		
		}).start();
    }
    
    public void updateConfig(){    		        
    	if(this.initialized){
    		if(this.buttons != null && this.buttons.length > 0){
    			for(int i = 0;i < this.buttons.length;i++){
    				if(this.buttons[i].getSelection()){
    					this.selected = (String)this.buttons[i].getData();
    				}
    			}
    		}
    		if(this.selected != null && !this.selected.equals(this.language)){
    			getConfig().setProperty(TGConfigKeys.LANGUAGE,this.selected);
    		}
    	}
    }    
    
    public void updateDefaults(){
    	if(this.initialized){
    		getConfig().setProperty(TGConfigKeys.LANGUAGE,getDefaults().getProperty(TGConfigKeys.LANGUAGE));
    	}
    }
    
    public void applyConfig(boolean force){    	    	
    	if(force || this.initialized){
    		boolean changed = force;    		
    		if(!changed && this.selected != null && (this.language == null || !this.selected.equals(this.language))){
    			changed = true;
    		}
    		if(changed){
				addSyncThread(new Runnable() {
					public void run() {
						TuxGuitar.instance().loadLanguage();
					}
				});
    		}
    	}
    }
}