package org.herac.tuxguitar.gui.system.plugins.base;

import org.herac.tuxguitar.gui.TuxGuitar;
import org.herac.tuxguitar.gui.system.plugins.TGPluginException;
import org.herac.tuxguitar.player.base.MidiPlayerException;
import org.herac.tuxguitar.player.base.MidiSequencerProvider;

public abstract class TGMidiSequencerProviderPlugin extends TGPluginBase{
	private boolean loaded;
	private MidiSequencerProvider provider;

	protected abstract MidiSequencerProvider getProvider();
	
	public void init(TuxGuitar instance) {
		this.provider = getProvider();
		this.loaded = false;
	}

	public void close() throws TGPluginException {
		try {
			this.provider.closeAll();
		} catch (MidiPlayerException e) {
			throw new TGPluginException(e.getMessage(),e);
		}
	}

	protected void addPluin()throws TGPluginException{
		if(!this.loaded){
			try {
				TuxGuitar.instance().getPlayer().addSequencerProvider(this.provider);
				this.loaded = true;
			} catch (MidiPlayerException e) {
				throw new TGPluginException(e.getMessage(),e);
			}
			
		}
	}
	
	protected void removePluin()throws TGPluginException{	
		if(this.loaded){
			try {
				TuxGuitar.instance().getPlayer().removeSequencerProvider(this.provider);
				this.loaded = false;
			} catch (MidiPlayerException e) {
				throw new TGPluginException(e.getMessage(),e);
			}
			
		}
	}	

	public void setEnabled(boolean enabled) throws TGPluginException{
		if(enabled){
			addPluin();
		}else{
			removePluin();
		}
	}	
}
